#!/usr/bin/python
'''
DDFacet, a facet-based radio imaging package
Copyright (C) 2013-2017  Cyril Tasse, l'Observatoire de Paris,
SKA South Africa, Rhodes University

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
'''

import subprocess
import os
import warnings
from setuptools import setup
from setuptools.command.install import install
from setuptools.command.sdist import sdist
from distutils.command.build import build
from os.path import join as pjoin
import sys

pkg='DDFacet'
skymodel_pkg='SkyModel'
build_root=os.path.dirname(__file__)

def get_version():
    # Versioning code here, based on
    # http://blogs.nopcode.org/brainstorm/2013/05/20/pragmatic-python-versioning-via-setuptools-and-git-tags/

    # Fetch version from git tags, and write to version.py.
    # Also, when git is not available (PyPi package), use stored version.py.
    version_py = os.path.join(build_root, pkg, 'version.py')

    try:
        version_git = subprocess.check_output(['git', 'describe', '--tags',
                                               '--abbrev=0']).rstrip()
    except:
        with open(version_py, 'r') as fh:
            version_git = open(version_py).read().strip().split('=')[-1].replace('"','')

    version_msg = "# Do not edit this file, pipeline versioning is governed by git tags"

    with open(version_py, 'w') as fh:
        fh.write(version_msg + os.linesep + "__version__=\"" + version_git +"\"")

    return version_git

def backend(compile_options):
    if compile_options is not None:
        print >> sys.stderr, "Compiling extension libraries with user defined options: '%s'"%compile_options
    path = pjoin(build_root, pkg, 'cbuild')
    try:
        subprocess.check_call(["mkdir", path])
    except:
        warnings.warn("%s already exists in your source folder. We will not create a fresh build folder, but you "
                      "may want to remove this folder if the configuration has changed significantly since the "
                      "last time you run setup.py" % path)
    subprocess.check_call(["cd %s && cmake %s .. && make" %
                           (path, compile_options if compile_options is not None else ""), ""], shell=True)

class custom_install(install):
    install.user_options = install.user_options + [
        ('compopts=', None, 'Any additional compile options passed to CMake')
    ]
    def initialize_options(self):
        install.initialize_options(self)
        self.compopts = None

    def run(self):
        backend(self.compopts)
        install.run(self)

class custom_build(build):
    build.user_options = build.user_options + [
        ('compopts=', None, 'Any additional compile options passed to CMake')
    ]
    def initialize_options(self):
        build.initialize_options(self)
        self.compopts = None

    def run(self):
        backend(self.compopts)
        build.run(self)

class custom_sdist(sdist):
    def run(self):
        bpath = pjoin(build_root, pkg, 'cbuild')
        if os.path.isdir(bpath):
            subprocess.check_call(["rm", "-rf", bpath])
        sdist.run(self)

def define_scripts():
    #these must be relative to setup.py according to setuputils
    return [os.path.join(pkg, script_name) for script_name in ['DDF.py', 'CleanSHM.py', 'MemMonitor.py', 'Restore.py', 'SelfCal.py']]

setup(name=pkg,
      version=get_version(),
      description='Facet-based radio astronomy continuum imager',
      url='http://github.com/cyriltasse/DDFacet',
      classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering :: Astronomy"],
      author='Cyril Tasse',
      author_email='cyril.tasse@obspm.fr',
      license='GNU GPL v2',
      cmdclass={'install': custom_install,
                'build': custom_build,
                'sdist': custom_sdist,
               },
      packages=[pkg, skymodel_pkg],
      install_requires=[
            "Cython == 0.25.2",
            "numpy == 1.13.0rc2",
            "SharedArray == 2.0.2",
            "Polygon2 == 2.0.8",
            "pyFFTW == 0.10.4",
            "nose == 1.3.7",
            "astropy == 1.3.3",
            "deap == 1.0.1", #invalid release tag on v1.0.2 refuse to use
            "ipdb == 0.10.3",
            "python-casacore == 2.1.2",
            "pyephem == 3.7.6.0",
            "numexpr == 2.6.2",
            "pyfits == 3.4",
            "matplotlib == 2.0.2",
            "scipy == 0.19.0",
            "astro-kittens == 0.3.3",
            "meqtrees-cattery == 1.5.1",
            "owlcat == 1.4.2",
            "astLib == 0.8.0",
            "psutil == 5.2.2",
            "astro-tigger == 1.3.5",
            "py-cpuinfo == 3.2.0",
      ],
      include_package_data=True,
      zip_safe=False,
      scripts=define_scripts()
)
